from pathlib import Path
import pandas as pd
import numpy as np
import rich_click as click

from ...exceptions import InvalidFileExtension
from ...utils import safe_load


def merge_command(files, output, **kwargs):
    verbose = kwargs.pop("verbose", False)
    tscol   = kwargs.pop("tscol", "timestamp_iso")

    # create an array of timestamp column names to try
    tscols = [tscol, "timestamp", "timestamp_local"]

    # make sure the extension is either a csv or feather format
    output = Path(output)
    if output.suffix not in (".csv", ".feather"):
        raise InvalidFileExtension("Invalid file extension")

    save_as_csv = True if output.suffix == ".csv" else False

    # concat everything in filepath
    if verbose:
        click.secho("Files to read: {}".format(files), fg='green')

    df = pd.DataFrame()
    with click.progressbar(files, label="Parsing files") as bar:
        for f in bar:
            tmp = safe_load(f)

            # check for the column name and set to best guess
            for c in tscols:
                if c in tmp.columns:
                    tscol = c
                    break
                
            if not tscol in tmp.columns:
                click.secho("Time tscol was not found in the file; skipping file.", fg='red')
                continue
                
            # convert the timestamp column to a pandas datetime
            if not pd.core.dtypes.common.is_datetime_or_timedelta_dtype(tmp[tscol]):
                tmp[tscol] = tmp[tscol].apply(lambda x: pd.to_datetime(x, errors='coerce'))
            
            # drop the bad rows
            tmp = tmp.dropna(how='any', subset=[tscol])

            # re-convert to timestamp in case it's not
            if not pd.core.dtypes.common.is_datetime_or_timedelta_dtype(tmp[tscol]):
                tmp[tscol] = tmp[tscol].apply(lambda x: pd.to_datetime(x, errors='raise'))
            
            # localize the timezone if needed
            tmp[tscol] = tmp[tscol].apply(lambda x: x.tz_localize("UTC") if not x.tzinfo else x)

            # set the index
            tmp.set_index(tscol, inplace=True)

            # merge with the other files
            df = pd.merge(df, tmp, left_index=True, right_index=True, how='outer')       

    # save the file
    if verbose:
        click.secho("Saving file to {}".format(output), fg='green')

    if save_as_csv:
        df.to_csv(output)
    else:
        df.reset_index().to_feather(output)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pandas as pd
import numpy as np
import h5py
from datetime import datetime, timedelta

## methods for wrangling PTR-TOF-MS Vocus hdf5 file data and compiling into some properties
def load_vocus_data(file, **kwargs):
    """
    extracts useful data from Vocus hdf5 file.
    :param file: hdf5 filepath
    :type file: str
    """
    metadata_ = kwargs.pop('metadata', False)

    with h5py.File(file, "r") as f:
        timestamps = get_times(f)
        mass_axis = get_mass_axis(f)
        tof_data = get_tof_data(f, len(timestamps), len(mass_axis))
        if metadata_:
            metadata = get_metadata(f)
            
    if metadata_:
        return timestamps, mass_axis, tof_data, metadata
    else:
        return timestamps, mass_axis, tof_data

def get_times(f):
    """
    extracts array of timestamps that match up with ToF data from Vocus file
    """
    # get experiment start time from log file and transform string to datetime
    start_time = f['AcquisitionLog']['Log']['timestring'][0]
    start_time = datetime.strptime(start_time.decode('UTF-8'), "%Y-%m-%dT%H:%M:%S+00:00")

    # times of observation are recorded as second offsets from start time
    buftimes = np.array(f['TimingData']['BufTimes'])
    
    timestamps = np.array([start_time + timedelta(seconds=i) for i in buftimes.reshape(-1)])
    
    # when the Vocus stops recording measurements, it still finishes out its last five second interval,
    # and records the time of the empty measurements as the start time. This code sets the empty measurement
    # times to NaN
    mask = (timestamps == timestamps[0])
    mask[0] = False

    timestamps[mask] = np.nan

    return timestamps

def get_tof_data(f, t, n):
    """
    extracts array of ToF data of shape (t, n) where
    t = number of snapshots that were taken during the experiment, or in other words the number of 
    timesteps (typically seconds) that the experiment was running
    n = number of mass bins that the mass spec is equipped to observe
    """
    tof_data = np.array(f['FullSpectra']['TofData'], dtype=np.uint16)
    tof_data = tof_data.reshape(t, n)
    return tof_data

def get_metadata(f):
    """
    returns array of metadata that corresponds to each measurement. 
    The metadata is currently stored in:
    TPS2
    ---> TEMP_INLET target [C]
    by Jordan
    """
    # times of observation are recorded as second offsets from start time
    buftimes = np.array(f['TimingData']['BufTimes'])

    metadata = np.array(f['TPS2']['TwData'])
    try:
        metadata_array = np.array(metadata[:, 87], dtype=np.uint16)
        return np.repeat(metadata_array, buftimes.shape[1])
    except:
        metadata_array = metadata[:,:,85].reshape(-1)
        return metadata_array

def get_mass_axis(f):
    """
    returns array of mass values that the ToF arrays correspond to
    """
    mass_axis = np.array(f['FullSpectra']['MassAxis'])
    return mass_axis



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path
import pandas as pd
import numpy as np
import rich_click as click
from os import path

from ...models import *
from ...utils import safe_load
from ...exceptions import InvalidFileExtension, InvalidArgument

def integrate_peaks_command(file, output, **kwargs):
    tscol = kwargs.pop('tscol', None)
    ignore = kwargs.pop('ignore', None)
    columns = kwargs.pop('columns', 'smiles')

    default_config_path = path.join(path.dirname(__file__), '../../config/peak-list.yml')
    # config = kwargs.pop('config', 'tofspec/config/peak-list.yml')
    config = kwargs.pop('config', default_config_path)
    peak_list = Path(config)
    if peak_list.suffix not in (".yml", ".yaml"):
        raise InvalidFileExtension("Invalid YAML file extension")

    # make sure the extension is either a csv or feather format
    output = Path(output)
    if output.suffix not in (".csv", ".feather"):
        raise InvalidFileExtension("Invalid output file extension")

    save_as_csv = True if output.suffix == ".csv" else False

    df = safe_load(file)

    timestamps = None
    metadata = None

    if tscol is not None:
        df[tscol] = pd.to_datetime(df[tscol])
        timestamps = df[tscol].to_numpy(dtype=np.datetime64)
        df.drop(tscol, axis=1, inplace=True)

    if ignore is not None:
        metadata = df[ignore].to_numpy()
        df.drop(ignore, axis=1, inplace=True)

    mass_axis = df.columns.to_numpy(dtype=np.float32)

    tof_data = df.to_numpy(dtype=np.float32)

    compound_df = time_series_df_from_yaml(tof_data, mass_axis, peak_list=config, columns=columns, timestamps=timestamps, metadata=metadata)

    # save the file
    click.secho("Saving file to {}".format(output), fg='green')

    if save_as_csv:
        compound_df.to_csv(output)
    else:
        compound_df.columns = compound_df.columns.astype(str)
        compound_df.reset_index().to_feather(output)






#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path
import pandas as pd
import numpy as np
import yaml
import rich_click as click

from ...utils import *
from ...exceptions import InvalidFileExtension, InvalidArgument

def config_command(file, output, **kwargs):
    #does file include a smiles column? -- defaults to False
    _smiles = kwargs.pop('smiles', False)

    #does file include an ion column? -- defaults to False and assumes that a mf column exists instead
    _ion = kwargs.pop('ion', False)

    #if user wants to specify a name or author
    name = kwargs.pop('name', None)
    author = kwargs.pop('author', None)

    #path to lookup table
    lookup_table = "tofspec/db/database.feather"

    # make sure the extension is either a csv or feather format
    file = Path(file)
    if file.suffix not in (".csv", ".feather"):
        raise InvalidFileExtension("Invalid input file extension")

    df = safe_load(file)
    df.columns = df.columns.str.lower()
    
    #check that input contains necessary mass range columns with correct names (case insensitive)
    if not pd.Series(['min', 'max']).isin(df.columns).all():
        raise InvalidArgument("FILE must contain columns: 'min', 'max' to describe the m/z integration range for each compound")
    #check that input contains either mf or smiles or both with correct names (case insensitive)
    if not pd.Series(['mf', 'ion', 'smiles']).isin(df.columns).any():
        raise InvalidArgument("FILE's columns must contain one of 'mf', 'smiles', 'ion' for functional grouping purposes")
    #check that the input contains a smiles column if the --smiles flag is passed

    if _smiles:
        if 'smiles' not in df.columns:
            raise InvalidArgument("If --smiles is passed, then FILE must contain a column named 'smiles'")
        # just parse the df and turn it into yaml
        data = df.to_dict('records')

        config_data = {'name':name,
                   'author': author,
                   'peak-list':data}
    else:
        #load functional group lookup table
        fx_df = pd.read_feather(lookup_table)
        if _ion:
            if 'ion' not in df.columns:
                raise InvalidArgument("If --ion is passed, then FILE must contain a column named 'ion'")
            df['mf'] = df['ion'].apply(deionize_regex)
        
        df = df.drop_duplicates(subset='mf', keep="last")

        # for now use the first SMILES found in the database depending on mf
        # we will also give the user an option to use all of the functional groups that correspond to a single isomer
        clean_df = df.merge(fx_df, how='inner', on='mf')[['mf','smiles', 'min', 'max']].drop_duplicates(subset='mf', keep="first").reset_index(drop=True)

        data = clean_df.to_dict('records')

        config_data = {'name':name,
                   'author': author,
                   'peak-list':data}
    
    # save the file
    click.secho("Saving file to {}".format(output), fg='green')

    write_yaml(output, config_data)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np


def find_indices(x, xlower, xupper):
    """
    Given an array and a value range, return the indices that most nearly match up with that value range.

    :param x: array of one-dimensional numeric data
    :type x: array-like
    :param xlower: lower bound
    :type xlower: int/float
    :param xupper: upper bound
    :type xupper: int/float
    :return indices: indices that match up with input value range
    :rtype: tuple
    """
    limits = np.array([xlower, xupper])
    indices = np.searchsorted(x, limits)

    try:
        candidates = x[np.stack((indices - 1, indices), axis=0)]
    except:
        return [0,0]
    indices += np.abs(candidates - limits).argmin(axis=0) - 1

    return indices

def integrate_peak(y, x, indices):
    """
    Integrate along the given axis in a certain x-range using the composite trapezoidal rule.

    :param y: Input array to integrate.
    :type y: array_like
    :param x: array of x-values corresponding to y
    :type x: array-like
    :param indices: indices that match up with input value range
    :type indices: tuple
    :return trapz: Definite integral of y = n-dimensional array as approximated along a
                 single axis by the trapezoidal rule.
    :rtype: float
    """

    s = slice(indices[0], indices[1] + 1)

    return np.trapz(y[:, s], x[s])


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pandas as pd
import numpy as np
from datetime import datetime, timedelta
import yaml
import rich_click as click
from itertools import chain

from .integrate import *
from .utils import *


def get_time_series(tof_data, mass_axis, mass, **kwargs):
    """
    Compute the abundance of a certain m/Q value vs time. If input is a single mass (m/Q) value,
    the bin for integration is assumed to have a width of 1. Otherwise, mass should be passed as a tuple
    of the form (mass_lower, mass_upper), and mass_range should be specified as True.

    Inputs
    ------
    :param tof_data: matrix of TOF mass spec data. shape = (t,n) where
                t = number of snapshots that were taken during the experiment, or in other words the number of 
                    timesteps (typically seconds) that the experiment was running
                n = number of TOF bins that the mass spec is equipped to observe
    :type tof_data: np.ndarray
    :param mass_axis: array of m/Q values that characterize the TOF bins of the mass spec
    :type mass_axis: np.ndarray
    :param mass: m/Q value to be integrated over (or m/Q range -- see optional args)
    :type mass: float or tuple

    Optional Arguments
    ------------------
    :param binsize: if mass is a single value, the width of the m/Q integration range. (default = 1)
    :type binsize: float
    :param mass_range: if True, then mass should be given as a tuple that gives the lower bound and upper bound
                        of the integration range. (default = False)
    :type mass_range: boolean

    Output
    ------
    :return: time series array of the integrated counts/concentration for the specified m/Q
    :rtype: np.array (shape=(t,))
    """
    binsize = kwargs.pop('binsize', 1)
    mass_range = kwargs.pop('mass_range', False)

    if mass_range == False:
        mass_lower = (mass-(binsize/2))
        mass_upper = (mass+(binsize/2))
    else:
        mass_lower = mass[0]
        mass_upper = mass[1]

    indices = find_indices(mass_axis, mass_lower, mass_upper)

    tof_time_series = integrate_peak(tof_data, mass_axis, indices)
    
    return tof_time_series



def get_time_series_df(tof_data, mass_axis, masses, **kwargs):
    """
    Using a list of masses or mass range tuples, create a time series 
    dataframe with a column for every mass in the list. Options to specify
    column names, supply timestamps and/or metadata.


    Inputs
    ------
    :param tof_data: matrix of TOF mass spec data. shape = (t,n) where
                t = number of snapshots that were taken during the experiment, or in other words the number of 
                    timesteps (typically seconds) that the experiment was running
                n = number of TOF bins that the mass spec is equipped to observe
    :type tof_data: np.ndarray
    :param mass_axis: array of m/Q values that characterize the TOF bins of the mass spec
    :type mass_axis: array-like
    :param masses: list/array of m/Q values to be integrated over (or m/Q range -- see optional args)
    :type mass: array-like (floats or tuples)

    Optional Arguments
    ------------------
    :param binsize: if mass is a single value, the width of the m/Q integration range. (default = 1)
    :type binsize: float
    :param mass_range: if True, then mass should be given as a tuple that gives the lower bound and upper bound
                        of the integration range. (default = False)
    :type mass_range: boolean
    :param names: identifiers for the list of masses provided. could be ions, compounds, or SMILES
    :type names: array-like
    :param timestamps: datetimes that match each row of data in tof_data
    :type timestamps: array-like
    :param metadata: metadata that match each row of data in tof_data
    :type metadata: array-like

    Output
    ------
    :return: dataframe of the integrated counts/concentration for the specified m/Q values
    :rtype: pd.Dataframe
    """
    binsize = kwargs.pop('binsize', 1)
    mass_range = kwargs.pop('mass_range', False)
    names = kwargs.pop('names', None)
    timestamps = kwargs.pop('timestamps', None)
    metadata = kwargs.pop('metadata', None)

    time_series_masses = []
    for m in masses:
        time_series_masses.append(get_time_series(tof_data, mass_axis, m, binsize=binsize, mass_range=mass_range))

    if names is None:
        for m in masses:
            names.append('m{} abundance'.format(m))

    if timestamps is not None:
        names.insert(0, "timestamp")
        time_series_masses.insert(0, timestamps)

    time_series_df = pd.DataFrame(dict(zip(names, time_series_masses)))

    if "timestamp" in time_series_df.columns:
        time_series_df['timestamp'] = pd.to_datetime(time_series_df['timestamp'])
        time_series_df = time_series_df.set_index('timestamp', drop=True)

    if metadata is not None:
        time_series_df['metadata'] = metadata

    return time_series_df


def time_series_df_from_yaml(tof_data, mass_axis, **kwargs):
    """
    Using the default mass list in the config/voc-db.yml file, or one 
    specified by the user, create a time series dataframe with a column for
    every compound in the list.

    Inputs
    ------
    :param tof_data: matrix of TOF mass spec data. shape = (t,n) where
                t = number of snapshots that were taken during the experiment, or in other words the number of 
                    timesteps (typically seconds) that the experiment was running
                n = number of TOF bins that the mass spec is equipped to observe
    :type tof_data: np.ndarray
    :param mass_axis: array of m/Q values that characterize the TOF bins of the mass spec
    :type mass_axis: array-like

    Optional Arguments
    ------------------
    :param peak_list: path to configuration yml file
    :type peak_list: str
    :param timestamps: array of datetimes that match the moments of observation
    :type timestamps: array-like
    :param metadata: array of metadata that match the observations
    :type metadata: array-like

    Output
    ------
    :return: dataframe of the integrated counts/concentration for the specified m/Q values
    :rtype: pd.Dataframe

    """
    timestamps = kwargs.pop('timestamps', None)
    metadata = kwargs.pop('metadata', None)
    peak_list = kwargs.pop('peak_list', 'config/peak-list.yml')
    columns = kwargs.pop('columns', 'smiles')
    voc_dict = read_yaml(peak_list)
    mf, smiles, min, max = peak_list_from_dict(voc_dict)
    if columns == 'mf':
        cols = mf
    else:
        cols = smiles
    masses = [list(x) for x in zip(min, max)]

    time_series_df = get_time_series_df(tof_data, mass_axis, masses, names=cols, mass_range=True, timestamps=timestamps, metadata=metadata)
    
    time_series_df = time_series_df.sort_index()

    return time_series_df


def group_time_series_df(time_series_df, **kwargs):
    """
    Based on the groups listed in the config/voc-db.yml file,
    sum the time series dataframe to have those groups as columns.

    Inputs
    ------
    :param time_series_df: dataframe of the integrated counts/concentration for the specified m/Q values
    :type time_series_df: pd.Dataframe
    :param peak_list: path to configuration yml file
    :type peak_list: str

    Optional Arguments
    ------------------
    :param lookup_table: path to SMILES / functional groups lookup table
    :type lookup_table: str
    :param columns: the column names can either be SMILES strings ('smiles') or molecular formulas ('mf') 
                    of different compounds. default = 'smiles'

    Output
    ------
    :return: dataframe of the integrated counts/concentration for the specified functional groups
    :rtype: pd.Dataframe
    """
    #path to lookup table
    lookup_table = kwargs.pop('lookup_table', 'db/database.feather')
    #method -- either use smiles or mf to do the integration
    columns = kwargs.pop('columns', 'smiles')
    if columns in ['mf', 'smiles']:
        pass
    else: 
        raise Exception("Only `mf` and `smiles` are accepted inputs for columns")

    #load functional group lookup table
    fx_df = pd.read_feather(lookup_table)
    group_list = list(fx_df.columns)
    group_list.remove('mf')
    group_list.remove('smiles')

    #isolate only the columns which are in time_series_df
    if columns == 'mf':
        mf = time_series_df.columns.to_numpy()
        subset_df = fx_df.loc[fx_df[columns].isin(mf)]
    else:
        smiles = time_series_df.columns.to_numpy()
        subset_df = fx_df.loc[fx_df[columns].isin(smiles)]

    #get all of the compounds in each group
    groups_smiles_dict = {} 
    for g in group_list:
        subset_g = subset_df.loc[subset_df[g] == 1][columns].to_list()
        groups_smiles_dict[g] = {columns: list(set(subset_g)),}

    #build grouped dataframe
    grouped_df = pd.DataFrame()
    grouped_df.index = time_series_df.index
    # grouped_df['metadata'] = time_series_df['metadata']

    for group in groups_smiles_dict.keys():
        grouped_df[group] = time_series_df[groups_smiles_dict[group][columns]].sum(axis=1)

    return grouped_df



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path
import pandas as pd
import numpy as np
import rich_click as click

from tofspec.models import group_time_series_df

from ...models import *
from ...utils import safe_load
from ...exceptions import InvalidFileExtension, InvalidArgument

def label_command(file, output, **kwargs):
    tscol = kwargs.pop('tscol', None)
    ignore = kwargs.pop('ignore', None)
    columns = kwargs.pop('columns', 'smiles')

    # make sure the extension is either a csv or feather format
    output = Path(output)
    if output.suffix not in (".csv", ".feather"):
        raise InvalidFileExtension("Invalid output file extension")

    save_as_csv = True if output.suffix == ".csv" else False

    df = safe_load(file)

    label_df = group_time_series_df(df, lookup_table='tofspec/db/database.feather', columns=columns)

    if tscol is not None:
        label_df[tscol] = pd.to_datetime(df[tscol])
    if ignore is not None:
        label_df[ignore] = df[ignore]

    # save the file
    click.secho("Saving file to {}".format(output), fg='green')

    if save_as_csv:
        label_df.to_csv(output)
    else:
        label_df.columns = label_df.columns.astype(str)
        label_df.reset_index().to_feather(output)


from rdkit import Chem
from rdkit.Chem import PandasTools 
import pandas as pd
import yaml
import numpy as np

def read_yaml(name):
    with open(name) as f:
        data = yaml.load(f, Loader=yaml.SafeLoader)
    return data

if __name__ == "main":
    df = pd.read_feather("database.feather")

    PandasTools.AddMoleculeColumnToFrame(df, smilesCol='smiles')

    #add all functional groups into the table
    substructures = read_yaml('substructures.yml')['substructures']

    for s in substructures:
        if s['name'] not in df.columns:
            # RDKit substructure matching
            df[s['name']] = df['ROMol'].map(lambda x: x.HasSubstructMatch(Chem.MolFromSmarts(s['smarts'])), na_action='ignore')
            # set to 1s and 0s rather than True / False
            df[s['name']] = df[s['name']].astype(int)
        else:
            pass
    
    # add BTEX column based on molecular formulas
    btex_formulas = ['C6H6', 'C7H8', 'C8H10']
    df['BTEX'] = np.where(df['mf'].isin(btex_formulas), 1, 0)

    #drop Mol column
    df = df.drop(['ROMol'], axis=1)

    #to feather
    df.to_feather("database.feather")


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path
import pandas as pd
import numpy as np
import h5py
import rich_click as click

from ...exceptions import InvalidFileExtension, InvalidArgument
from ...load import *

def load_command(file, output, **kwargs):
    instrument = kwargs.pop('instrument', 'vocus')
    file_format = kwargs.pop('format', 'h5')
    metadata_ = kwargs.pop('metadata', False)
    
    # right now we can only read Vocus files
    if instrument != 'vocus' or file_format != 'h5':
        raise InvalidArgument("There is currently only loading support for .h5 files from the TOFWERK PTR-TOF-MS Vocus instrument")

    # make sure the extension is either a csv or feather format
    output = Path(output)
    if output.suffix not in (".csv", ".feather"):
        raise InvalidFileExtension("Invalid output file extension")

    save_as_csv = True if output.suffix == ".csv" else False

    #load Vocus data
    if metadata_:
        timestamps, mass_axis, tof_data, metadata = load_vocus_data(file, metadata=metadata_)
        #assemble dataframe format
        df = pd.DataFrame(tof_data, columns=mass_axis)
        df['timestamp'] = timestamps
        df['metadata'] = metadata
    else:
        timestamps, mass_axis, tof_data = load_vocus_data(file, metadata=metadata_)
        #assemble dataframe format
        df = pd.DataFrame(tof_data, index=pd.DatetimeIndex(timestamps), columns=mass_axis)
        df['timestamp'] = timestamps

    # save the file
    click.secho("Saving file to {}".format(output), fg='green')

    if save_as_csv:
        df.to_csv(output)
    else:
        df.columns = df.columns.astype(str)
        df.reset_index().to_feather(output)

    
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pandas as pd
import numpy as np
import yaml
import re
from pathlib import Path

from .exceptions import InvalidFileExtension

def safe_load(fpath, **kwargs):
    """Load and return a file
    """
    p = Path(fpath)

    if p.suffix == ".csv":
        as_csv = True
    elif p.suffix == ".feather":
        as_csv = False
    else:
        raise InvalidFileExtension

    sample = pd.read_csv(fpath, nrows=1) if as_csv else pd.read_feather(fpath)
    dtypes = sample.dtypes # Get the dtypes
    cols = sample.columns # Get the columns

    dtype_dictionary = {} 
    for c in cols:
        if str(dtypes[c]) == 'float64':
            dtype_dictionary[c] = 'float32'
        else:
            dtype_dictionary[c] = str(dtypes[c])

    tmp = pd.read_csv(fpath, dtype=dtype_dictionary) if as_csv else pd.read_feather(fpath)
    

    # drop the extra column if it was added
    if "Unnamed: 0" in tmp.columns:
        del tmp["Unnamed: 0"]

    return tmp


## reading YAML config files

def write_yaml(name, data):
    with open(name, 'w') as f:
        documents = yaml.dump(data, f, indent=4, default_flow_style=False, sort_keys=False)

def read_yaml(name):
    with open(name) as f:
        data = yaml.load(f, Loader=yaml.SafeLoader)
    return data

def build_peak_list(mf, smiles, min, max, **kwargs):
    name = kwargs.pop('name', None)
    author = kwargs.pop('author', None)

    data = {'name':name,
            'author': author,
            'peak-list':[{
                'mf': mf[i],
                'smiles': smiles[i],
                'mass-range':{
                    'min':min[i],
                    'max':max[i],
                }}for i in range(len(smiles))
                ]}
    
    return data

def peak_list_from_dict(data):
    mf = []
    smiles = []
    min = []
    max = []
    for i in data['peak-list']:
        mf.append(i['mf'])
        smiles.append(i['smiles'])
        min.append(i['mass-range']['min'])
        max.append(i['mass-range']['max'])
    return mf, smiles, min, max

def deionize_regex(ion):
    """Remove a hydrogen and deionize a molecular formula using regex
    """
    #make uppercase
    ion = ion.upper()

    # check if H is in the formula
    # if so, subtract one H from the formula after deionizing
    if 'H' in ion:
        # remove +/- to deionize formula
        clean_mf = re.sub(r'[^\w]', '', ion)
        try:
            # get the number of Hs in the ion formula
            num_Hs = re.search('H(\d+)', ion, re.IGNORECASE).group(1)
            # subtract 1 H
            deionized_Hs = int(num_Hs)-1
            # if there is a single H left remove the subscript number
            if deionized_Hs == 1:
                replacement = 'H'
            # if there are more than a single H left, replace the subscript number
            else:
                replacement = 'H'+str(deionized_Hs)
            deionized_mf = re.sub('H'+num_Hs, replacement, clean_mf)
            return deionized_mf
        # the try statement will fail if there is a single H in the ion formula
        # this exception deals with removing that single H
        except:
            deionized_mf = re.sub('H', '', clean_mf)
            return deionized_mf
    else:
        # remove +/- to deionize formula
        clean_mf = re.sub(r'[^\w]', '', ion)
        return clean_mf


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['tofspec', 'tofspec.cli', 'tofspec.cli.commands', 'tofspec.db']

package_data = \
{'': ['*'], 'tofspec': ['config/*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'click>=8.0.4,<9.0.0',
 'dask>=2022.4.1,<2023.0.0',
 'h5py>=3.6.0,<4.0.0',
 'numpy>=1.22.3,<2.0.0',
 'pandas>=1.4.1,<2.0.0',
 'pyarrow>=7.0.0,<8.0.0',
 'rich-click>=1.5.1,<2.0.0']

entry_points = \
{'console_scripts': ['tofspec = tofspec.cli:main']}

setup_kwargs = {
    'name': 'py-tofspec',
    'version': '0.1.21',
    'description': '',
    'long_description': 'None',
    'author': 'Joe Palmo',
    'author_email': 'jpalmo21@amherst.edu',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path
import pandas as pd
import numpy as np
import rich_click as click

from ...exceptions import InvalidFileExtension
from ...utils import safe_load


def concat_command(files, output, **kwargs):

    # make sure the extension is csv
    output = Path(output)
    if output.suffix not in (".csv", ".feather"):
        raise InvalidFileExtension("Invalid file extension")

    save_as_csv = True if output.suffix == ".csv" else False

    # concat everything in filepath
    click.secho("Files to read: {}".format(files), fg='green')

    # read all files
    data = []
    with click.progressbar(files, label="Parsing files") as bar:
        for f in bar:
            tmp = safe_load(f)

            data.append(tmp)

    # concat all of the files together
    df = pd.concat(data, sort=False)

    #sorting based on timestamp column
    df['timestamp'] = pd.to_datetime(df['timestamp'])
    df = df.set_index('timestamp', drop=True)
    df = df.sort_index()

    if df.empty:
        raise Exception("No data")

    # save the file
    click.secho("Saving file to {}".format(output), fg='green')

    if save_as_csv:
        df.to_csv(output)
    else:
        df.reset_index().to_feather(output)



import rich_click as click
import pkg_resources

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

__version__ = pkg_resources.get_distribution('py-tofspec').version

#set up cli tool
@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(__version__)
@click.pass_context
def main(ctx):
    pass

#add concat command
@click.command("concat", short_help="concatenate files together")
@click.argument("files", nargs=-1, type=click.Path())
@click.option("-o", "--output", default="output.csv", help="The filepath where you would like to save the file", type=str)
def concat(files, output, **kwargs):
    """Concatenate FILES together and save to OUTPUT.
    FILES is the collection or list of files that you are concatenating together. They 
    can be provided as a list or by using a wildcard and providing the path with wildcard.
    """
    from .commands.concat import concat_command

    concat_command(files, output, **kwargs)

@click.command("merge", short_help="merge two files together on their timestamp")
@click.argument("files", nargs=-1, type=click.Path())
@click.option("-ts", "--tscol", default="timestamp", help="The column by which to join the files", type=str)
@click.option("-o", "--output", default="output.csv", help="The filepath where you would like to save the file", type=str)
@click.option("-v", "--verbose", is_flag=True, help="Enable verbose mode (debugging)")
def merge(files, tscol, output, verbose, **kwargs):
    """Merge FILES together and save to OUTPUT.
    """
    from .commands.merge import merge_command

    merge_command(files, output, tscol=tscol, verbose=verbose, **kwargs)

#add config command
@click.command("config", short_help="build peak list from .csv")
@click.argument("file", nargs=1, type=click.Path())
@click.option("-s", "--smiles", is_flag=True, default=False, help="Does FILE have a smiles column?")
@click.option("-ion", "--ion", is_flag=True, default=False, help="Does FILE have an ion column instead of a mf column?")
@click.option("-n", "--name", default=None, help="Name for the peak list", type=str)
@click.option("-a", "--author", default=None, help="Author of the peak list", type=str)
@click.option("-o", "--output", default="output.yaml", help="The filepath where you would like to save the peak list", type=str)
def config(file, output, **kwargs):
    """Use FILE to construct a peak list configuration .yaml file that will be used to integrate peaks
    and label / group data.
    """
    from .commands.config import config_command

    config_command(file, output, **kwargs)


#add load command
@click.command("load", short_help="parse raw mass spec data files")
@click.argument("file", nargs=1, type=click.Path())
@click.option("-i", "--instrument", default="vocus", help="The instrument that FILE comes from", type=str)
@click.option("-f", "--format", default="h5", help="The format/file extension of FILE", type=str)
@click.option("-m", "--metadata", is_flag=True, default=False, help="Does the file include metadata?")
@click.option("-o", "--output", default="output.csv", help="The filepath where you would like to save the file", type=str)
def load(file, output, **kwargs):
    """Parse mass spec FILE and save relevant data to OUTPUT.
    Read TOF data matrix from FILE. The structure of FILE is determined by the 
    optional --instrument and --format arguments. Currently, only 'vocus' and 'h5' are accepted.
    """
    from .commands.load import load_command

    load_command(file, output, **kwargs)

#add integrate peaks command
@click.command("integrate-peaks", short_help="integrate ion peaks in raw mass spec data")
@click.argument("file", nargs=1, type=click.Path())
@click.option("-c", "--config", default="tofspec/config/peak-list.yml", help="The peak list .yml file that guides the integration process", type=click.Path())
@click.option("-ts", "--tscol", help="Column in FILE which contains timestamps")
@click.option("-i", "--ignore", help="Names of metadata column(s) which should be ignored in the integration and passed to OUTPUT untouched")
@click.option("-col", "--columns", type=click.Choice(['smiles', 'mf'], case_sensitive=False),  default='smiles', help="Choose either molecular formula (`mf`) or SMILES string (`smiles`) as the column names of OUTPUT")
@click.option("-o", "--output", default="output.csv", help="The filepath where you would like to save the file", type=str)
def integrate_peaks(file, output, **kwargs):
    """Convert FILE, a matrix of raw PTR-TOF-MS data (TOF bins X timestamps) to a time series of
         integrated ion counts/concentrations for ions specified in the peak list (CONFIG).
    """
    from .commands.integrate_peaks import integrate_peaks_command

    integrate_peaks_command(file, output, **kwargs)

@click.command("label", short_help="sum compounds counts/concentrations by substructure")
@click.argument("file", nargs=1, type=click.Path())
@click.option("-ts", "--tscol", help="column in FILE which contains timestamps")
@click.option("-i", "--ignore", help="names of metadata column(s) which should not be included in the integration but should be passed to OUTPUT")
@click.option("-col", "--columns", type=click.Choice(['smiles', 'mf'], case_sensitive=False),  default='smiles', help="choose either molecular formula (`mf`) or SMILES string (`smiles`) as the column names of FILE")
@click.option("-o", "--output", default="output.csv", help="The filepath where you would like to save the file", type=str)
def label(file, output, **kwargs):
    """Convert FILE, a matrix of compound counts/concentrations, to a time series of integrated
        substructure/functional group concentrations. For more info on how to choose different substructures...
    """
    from .commands.label import label_command

    label_command(file, output, **kwargs)


#add all commands
main.add_command(concat)
main.add_command(merge)
main.add_command(config)
main.add_command(load)
main.add_command(integrate_peaks)
main.add_command(label)
